package CFDateDropDownField::Plugin;
use strict;

use PowerCMS::Util qw( build_tmpl valid_ts );

our $plugin = MT->component( 'CFDateDropDownField' );

sub _cb_tp_edit_contactform {
    my ( $cb, $app, $param, $tmpl ) = @_;
    $param->{ js_include } .= <<'EOF';
<script type="text/javascript">
(function(j) {
  var typeOfDateDropDown = {
    date_dropdown: {
      visibility: {
        opt: true,
        size: false,
        def: true,
        validate: true,
        length: false
      },
      defaultType: 'text'
    }
  };
  j.contactForm.addType(typeOfDateDropDown);
})(jQuery);
</script>
EOF
}

sub _cb_contactform_pre_build_date_dropdown {
    my ( $cb, $app, $mtml, $args, $params ) = @_;
    my $ctx = $args->{ ctx };
    my $contactform = $ctx->stash( 'contactform' );
    my $basename = $contactform->basename;
    my $default = build_tmpl( $app, $contactform->default, $args, $params );
    if ( $default ) {
        $default =~ s/,//g;
    }
    my ( $default_year, $default_month, $default_day );
    my $set_default;
    if ( ref( $app ) eq 'MT::App::ContactForm' ) {
        if ( $app->mode eq 'confirm' ) {
            $default_year = $app->param( $basename . '-year' );
            $default_month = $app->param( $basename . '-month' );
            $default_day = $app->param( $basename . '-day' );
            $set_default = 1;
            if ( $app->param( $basename ) eq '1' ) {
                if (! _format_date_dropdown( $app, $contactform ) ) {
                    $params->{ field_error } = 'required';
                }
            }
        }
    }
    if (! $set_default ) {
        if ( $default ) {
            $default_year = substr( $default, 0, 4 );
            $default_month = substr( $default, 4, 2 );
            $default_day = substr( $default, 6, 2 );
        }
    }
    my @field_loop_year;
    my @field_loop_month;
    my @field_loop_day;
    push ( @field_loop_year, { 'option_value' => '' } );
    push ( @field_loop_month, { 'option_value' => '' } );
    push ( @field_loop_day, { 'option_value' => '' } );
    for ( 1 .. 12 ) {
        my $month = sprintf( "%02d", $_ );
        push ( @field_loop_month, { 'option_value' => $month,
                                    'option_default' => ( $month == $default_month ) } );
    }
    for ( 1 .. 31 ) {
        my $day = sprintf( "%02d", $_ );
        push ( @field_loop_day, { 'option_value' => $day,
                                  'option_default' => ( $day == $default_day ) } );
    }
    my $options = build_tmpl( $app, $contactform->options, $args, $params );
    my @years = split( /,/, $options );
    if ( ( scalar @years ) == 2 ) {
        for ( $years[ 0 ] .. $years[ 1 ] ) {
            push ( @field_loop_year, { 'option_value' => $_,
                                       'option_default' => ( $_ == $default_year ) } );
        }
    } else {
        for my $year( @years ) {
            push ( @field_loop_year, { 'option_value' => $year,
                                       'option_default' => ( $year == $default_year ) } );
        }
    }
    $params->{ 'field_loop-year' } = \@field_loop_year;
    $params->{ 'field_loop-month' } = \@field_loop_month;
    $params->{ 'field_loop-day' } = \@field_loop_day;
}

sub _format_date_dropdown {
    my ( $app, $contactform ) = @_;
    my $basename = $contactform->basename;
    my $hidden = $app->param( $basename );
    if ( $hidden ) {
        if ( valid_ts( $hidden ) ) {
            return $hidden;
        } else {
            $app->param( $basename, undef );
        }
    }
    my $year = $app->param( $basename . '-year' );
    my $month = $app->param( $basename . '-month' );
    my $day = $app->param( $basename . '-day' );
    if ( $month && $month =~ /^[0-9]{1}$/ ) {
        $month = '0' . $month;
    }
    if ( $day && $day =~ /^[0-9]{1}$/ ) {
        $month = '0' . $month;
    }
    if ( ( $year =~ /^[0-9]{4}$/ ) && ( $month =~ /^[0-9]{2}$/ ) && ( $day =~ /^[0-9]{2}$/ ) ) {
        my $ts = $year . $month . $day . '000000';
        return $ts;
    }
    return '';
}

sub _validate_date_dropdown {
    if ( my $ts = _format_date_dropdown( @_ ) ) {
        if ( valid_ts( $ts ) ) {
            return 1;
        }
    }
    return 0;
}

1;